//	OptionsFile.h - Simple class used to save/restore options.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef OPTIONSFILE_H_
#define OPTIONSFILE_H_

#include <string>
#include <map>

///	Simple class used to save/restore options.
class OptionsFile
{
  public:
	///	Constructor.
	/*!
		If it exists, the file will be loaded here.
	 */
	OptionsFile();
	///	Destructor.
	/*!
		The file will be saved here.
	 */
	~OptionsFile();

	///	Sets the value of the named option (float).
	void setOption(const std::wstring& option, float value);
	///	Sets the value of the named option (int).
	void setOption(const std::wstring& option, int value);
	///	Sets the value of the named option (string).
	void setOption(const std::wstring& option, const::std::wstring& value);

	///	Returns the value of the named option (float).
	/*!
		\return -1 if the option does not exist.
	 */
	float getOptionFloat(const std::wstring& option) const;
	///	Returns the value of the named option (int).
	/*!
		\return -1 if the option does not exist.
	 */
	int getOptionInt(const std::wstring& option) const;
	///	Returns the value of the named option (string).
	const std::wstring getOptionString(const std::wstring& option) const;
  private:
	std::map<std::wstring, std::wstring> options;
};

#endif
