//	OptionsScreen.cpp - The title screen state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ScreamingSnakeballGlobalData.h"
#include "sound/PortAudioDevice.h"
#include "SnakeballSounder.h"
#include "drawing/Drawer.h"
#include "OptionsScreen.h"
#include "Application.h"

#include <sstream>

//------------------------------------------------------------------------------
GameStateCreator<OptionsScreen> OptionsScreen::creator;

//------------------------------------------------------------------------------
OptionsScreen::OptionsScreen(Application& app, SounderPtr& sound):
GameState(app, sound),
entryTween0(entryPos[0], entryPos[0], 0.1f),
entryTween1(entryPos[1], entryPos[1], 0.1f),
entryTween2(entryPos[2], entryPos[2], 0.1f),
entryTween3(entryPos[3], entryPos[3], 0.1f),
entryTween4(entryPos[4], entryPos[4], 0.1f),
entryTween5(entryPos[5], entryPos[5], 0.1f),
entryTween6(entryPos[6], entryPos[6], 0.1f),
entryTween7(entryPos[7], entryPos[7], 0.1f),
selectedEntry(0),
lastSelected(0),
screenShakeSlider(TwoFloats(760.0f, 160.0f), ThreeFloats(1.0f, 0.5f, 0.5f)),
backgroundFlashSlider(TwoFloats(760.0f, 232.0f), ThreeFloats(0.8125f, 0.5f, 0.625f)),
borderFlashSlider(TwoFloats(760.0f, 302.0f), ThreeFloats(0.75f, 0.5f, 0.75f)),
matchLengthSlider(TwoFloats(760.0f, 372.0f), ThreeFloats(0.52734375f, 0.5f, 1.0f)),
audioAPITween(audioAPIText.position, audioAPIText.position, 0.1f),
redInputTween(redInputText.position, redInputText.position, 0.1f),
blueInputTween(blueInputText.position, blueInputText.position, 0.1f),
joyRepeat(0),
redMeter(TwoFloats(62.0f, 11.0f), 1.0f),
blueMeter(TwoFloats(961.0f, 11.0f), -1.0f),
sounder(dynamic_pointer_cast<SnakeballSounder>(sound))
{
	int i;
	std::wstring storedAPI;
	std::wstringstream tempstr;
	bool availableInputDevice = false;
	AudioDevicePtr& audioDevice = getApplication().getAudioDevice();
	const std::vector<std::wstring>& audioInputDevices = audioDevice->getAvailableInputDevices();

	globalData = dynamic_pointer_cast<ScreamingSnakeballGlobalData>(getGlobalData());

	storedAPI = globalData->getAudioAPI();
	inputIndex = globalData->getAudioInputDevice();
	if((storedAPI != L"") && (storedAPI != audioDevice->getAPI()))
	{
		audioDevice->stop();
		audioDevice->setAPI(storedAPI);

		if(audioDevice->getAvailableInputDevices().size() > 0)
			availableInputDevice = true;

		audioAPI = audioDevice->getAPI();
		
		if(availableInputDevice)
			sounder->setNumChannels(audioDevice->getChannelsForInputDevice(inputIndex));
		else
			sounder->setNumChannels(0);

		tempstr << L"< " << audioAPI << L" >";
		audioAPIText.text = tempstr.str();
	
		if(availableInputDevice)
		{
			audioDevice->setDevices(audioDevice->getAvailableInputDevices()[inputIndex],
									L"Default");
			audioDevice->setNumChannels(audioDevice->getChannelsForInputDevice(inputIndex), 2);
		}
		else
		{
			audioDevice->setDevices(L"Default", L"Default");
			audioDevice->setNumChannels(0, 2);
		}
		audioDevice->start();
	}

	redChannel = globalData->getRedInputChannel();
	blueChannel = globalData->getBlueInputChannel();

	audioAPI = audioDevice->getAPI();
	updateRedText();
	updateBlueText();

	audioAPIText.position = TwoFloats(760.0f, 442.0f);
	audioAPIText.colour = ThreeFloats(0.5f, 0.5625f, 1.0f);
	audioAPIText.text = audioAPI;
	audioAPIText.font = L"TimeFont";
	audioAPITween.setIndex(1.0f);

	redInputText.position = TwoFloats(760.0f, 512.0f);
	redInputText.colour = ThreeFloats(0.5f, 0.5625f, 1.0f);
	redInputText.text = redInput;
	redInputText.font = L"TimeFont";
	redInputTween.setIndex(1.0f);

	blueInputText.position = TwoFloats(760.0f, 582.0f);
	blueInputText.colour = ThreeFloats(0.5f, 0.5625f, 1.0f);
	blueInputText.text = blueInput;
	blueInputText.font = L"TimeFont";
	blueInputTween.setIndex(1.0f);

	for(i=0;i<8;++i)
	{
		entries[i][0].colour = ThreeFloats(1.0f, 0.5f, 0.5f);
		entries[i][0].text = L"Screen Shake:";
		entries[i][1].colour = ThreeFloats(0.8125f, 0.5f, 0.625f);
		entries[i][1].text = L"Background Flash:";
		entries[i][2].colour = ThreeFloats(0.75f, 0.5f, 0.75f);
		entries[i][2].text = L"Border Flash:";
		entries[i][3].colour = ThreeFloats(0.52734375f, 0.5f, 1.0f);
		entries[i][3].text = L"Match Length:";
		entries[i][4].colour = ThreeFloats(0.5f, 0.5625f, 1.0f);
		entries[i][4].text = L"Audio API:";
		entries[i][5].colour = ThreeFloats(0.390625f, 0.70703125f, 1.0f);
		entries[i][5].text = L"Red Audio Input:";
		entries[i][6].colour = ThreeFloats(0.421875f, 0.8359375f, 0.796875f);
		entries[i][6].text = L"Blue Audio Input:";
		entries[i][7].colour = ThreeFloats(0.61328125f, 0.88671875f, 0.65234375f);
		entries[i][7].text = L"Back";
	}

	entries[0][0].position = TwoFloats(70.0f, 160.0f);
	entries[0][0].font = L"ScoreFont";
	entries[0][1].position = TwoFloats(70.0f, 232.0f);
	entries[0][1].font = L"TimeFont";
	entries[0][2].position = TwoFloats(70.0f, 302.0f);
	entries[0][2].font = L"TimeFont";
	entries[0][3].position = TwoFloats(70.0f, 372.0f);
	entries[0][3].font = L"TimeFont";
	entries[0][4].position = TwoFloats(70.0f, 442.0f);
	entries[0][4].font = L"TimeFont";
	entries[0][5].position = TwoFloats(70.0f, 512.0f);
	entries[0][5].font = L"TimeFont";
	entries[0][6].position = TwoFloats(70.0f, 582.0f);
	entries[0][6].font = L"TimeFont";
	entries[0][7].position = TwoFloats(70.0f, 652.0f);
	entries[0][7].font = L"TimeFont";

	entries[1][0].position = TwoFloats(70.0f, 160.0f-40.0f);
	entries[1][0].font = L"TimeFont";
	entries[1][1].position = TwoFloats(70.0f, 232.0f-8.0f);
	entries[1][1].font = L"ScoreFont";
	entries[1][2].position = TwoFloats(70.0f, 302.0f);
	entries[1][2].font = L"TimeFont";
	entries[1][3].position = TwoFloats(70.0f, 372.0f);
	entries[1][3].font = L"TimeFont";
	entries[1][4].position = TwoFloats(70.0f, 442.0f);
	entries[1][4].font = L"TimeFont";
	entries[1][5].position = TwoFloats(70.0f, 512.0f);
	entries[1][5].font = L"TimeFont";
	entries[1][6].position = TwoFloats(70.0f, 582.0f);
	entries[1][6].font = L"TimeFont";
	entries[1][7].position = TwoFloats(70.0f, 652.0f);
	entries[1][7].font = L"TimeFont";

	entries[2][0].position = TwoFloats(70.0f, 160.0f-40.0f);
	entries[2][0].font = L"TimeFont";
	entries[2][1].position = TwoFloats(70.0f, 232.0f-48.0f);
	entries[2][1].font = L"TimeFont";
	entries[2][2].position = TwoFloats(70.0f, 302.0f-8.0f);
	entries[2][2].font = L"ScoreFont";
	entries[2][3].position = TwoFloats(70.0f, 372.0f);
	entries[2][3].font = L"TimeFont";
	entries[2][4].position = TwoFloats(70.0f, 442.0f);
	entries[2][4].font = L"TimeFont";
	entries[2][5].position = TwoFloats(70.0f, 512.0f);
	entries[2][5].font = L"TimeFont";
	entries[2][6].position = TwoFloats(70.0f, 582.0f);
	entries[2][6].font = L"TimeFont";
	entries[2][7].position = TwoFloats(70.0f, 652.0f);
	entries[2][7].font = L"TimeFont";

	entries[3][0].position = TwoFloats(70.0f, 160.0f-40.0f);
	entries[3][0].font = L"TimeFont";
	entries[3][1].position = TwoFloats(70.0f, 232.0f-48.0f);
	entries[3][1].font = L"TimeFont";
	entries[3][2].position = TwoFloats(70.0f, 302.0f-48.0f);
	entries[3][2].font = L"TimeFont";
	entries[3][3].position = TwoFloats(70.0f, 372.0f-8.0f);
	entries[3][3].font = L"ScoreFont";
	entries[3][4].position = TwoFloats(70.0f, 442.0f);
	entries[3][4].font = L"TimeFont";
	entries[3][5].position = TwoFloats(70.0f, 512.0f);
	entries[3][5].font = L"TimeFont";
	entries[3][6].position = TwoFloats(70.0f, 582.0f);
	entries[3][6].font = L"TimeFont";
	entries[3][7].position = TwoFloats(70.0f, 652.0f);
	entries[3][7].font = L"TimeFont";

	entries[4][0].position = TwoFloats(70.0f, 160.0f-40.0f);
	entries[4][0].font = L"TimeFont";
	entries[4][1].position = TwoFloats(70.0f, 232.0f-48.0f);
	entries[4][1].font = L"TimeFont";
	entries[4][2].position = TwoFloats(70.0f, 302.0f-48.0f);
	entries[4][2].font = L"TimeFont";
	entries[4][3].position = TwoFloats(70.0f, 372.0f-48.0f);
	entries[4][3].font = L"TimeFont";
	entries[4][4].position = TwoFloats(70.0f, 442.0f-8.0f);
	entries[4][4].font = L"ScoreFont";
	entries[4][5].position = TwoFloats(70.0f, 512.0f);
	entries[4][5].font = L"TimeFont";
	entries[4][6].position = TwoFloats(70.0f, 582.0f);
	entries[4][6].font = L"TimeFont";
	entries[4][7].position = TwoFloats(70.0f, 652.0f);
	entries[4][7].font = L"TimeFont";

	entries[5][0].position = TwoFloats(70.0f, 160.0f-40.0f);
	entries[5][0].font = L"TimeFont";
	entries[5][1].position = TwoFloats(70.0f, 232.0f-48.0f);
	entries[5][1].font = L"TimeFont";
	entries[5][2].position = TwoFloats(70.0f, 302.0f-48.0f);
	entries[5][2].font = L"TimeFont";
	entries[5][3].position = TwoFloats(70.0f, 372.0f-48.0f);
	entries[5][3].font = L"TimeFont";
	entries[5][4].position = TwoFloats(70.0f, 442.0f-48.0f);
	entries[5][4].font = L"TimeFont";
	entries[5][5].position = TwoFloats(70.0f, 512.0f-8.0f);
	entries[5][5].font = L"ScoreFont";
	entries[5][6].position = TwoFloats(70.0f, 582.0f);
	entries[5][6].font = L"TimeFont";
	entries[5][7].position = TwoFloats(70.0f, 652.0f);
	entries[5][7].font = L"TimeFont";

	entries[6][0].position = TwoFloats(70.0f, 160.0f-40.0f);
	entries[6][0].font = L"TimeFont";
	entries[6][1].position = TwoFloats(70.0f, 232.0f-48.0f);
	entries[6][1].font = L"TimeFont";
	entries[6][2].position = TwoFloats(70.0f, 302.0f-48.0f);
	entries[6][2].font = L"TimeFont";
	entries[6][3].position = TwoFloats(70.0f, 372.0f-48.0f);
	entries[6][3].font = L"TimeFont";
	entries[6][4].position = TwoFloats(70.0f, 442.0f-48.0f);
	entries[6][4].font = L"TimeFont";
	entries[6][5].position = TwoFloats(70.0f, 512.0f-48.0f);
	entries[6][5].font = L"TimeFont";
	entries[6][6].position = TwoFloats(70.0f, 582.0f-8.0f);
	entries[6][6].font = L"ScoreFont";
	entries[6][7].position = TwoFloats(70.0f, 652.0f);
	entries[6][7].font = L"TimeFont";

	entries[7][0].position = TwoFloats(70.0f, 160.0f-40.0f);
	entries[7][0].font = L"TimeFont";
	entries[7][1].position = TwoFloats(70.0f, 232.0f-48.0f);
	entries[7][1].font = L"TimeFont";
	entries[7][2].position = TwoFloats(70.0f, 302.0f-48.0f);
	entries[7][2].font = L"TimeFont";
	entries[7][3].position = TwoFloats(70.0f, 372.0f-48.0f);
	entries[7][3].font = L"TimeFont";
	entries[7][4].position = TwoFloats(70.0f, 442.0f-48.0f);
	entries[7][4].font = L"TimeFont";
	entries[7][5].position = TwoFloats(70.0f, 512.0f-48.0f);
	entries[7][5].font = L"TimeFont";
	entries[7][6].position = TwoFloats(70.0f, 582.0f-48.0f);
	entries[7][6].font = L"TimeFont";
	entries[7][7].position = TwoFloats(70.0f, 652.0f-8.0f);
	entries[7][7].font = L"ScoreFont";

	for(i=0;i<8;++i)
		entryPos[i] = entries[0][i].position;

	entryTween0.setIndex(1.0f);
	entryTween1.setIndex(1.0f);
	entryTween2.setIndex(1.0f);
	entryTween3.setIndex(1.0f);
	entryTween4.setIndex(1.0f);
	entryTween5.setIndex(1.0f);
	entryTween6.setIndex(1.0f);
	entryTween7.setIndex(1.0f);

	screenShakeSlider.setValue(globalData->getScreenShake());
	screenShakeSlider.setBig(true);
	backgroundFlashSlider.setValue(globalData->getBackgroundFlash());
	backgroundFlashSlider.setBig(false);
	borderFlashSlider.setValue(globalData->getBorderFlash());
	borderFlashSlider.setBig(false);
	matchLengthSlider.setValue((globalData->getMatchLength()-1.0f)/14.0f);
	matchLengthSlider.setBig(false);
}

//------------------------------------------------------------------------------
OptionsScreen::~OptionsScreen()
{

}

//------------------------------------------------------------------------------
void OptionsScreen::update()
{
	if(lastJoyVal != 0.0f)
	{
		if(!joyRepeat)
		{
			joyRepeat = 15;

			if(fabsf(lastJoyVal.x) > fabsf(lastJoyVal.y))
			{
				if(lastJoyVal.x > 0.0f)
				{
					switch(selectedEntry)
					{
						case 0:
							screenShakeSlider.incValue(0.1f);
							globalData->setScreenShake(screenShakeSlider.getValue());
							break;
						case 1:
							backgroundFlashSlider.incValue(0.1f);
							globalData->setBackgroundFlash(backgroundFlashSlider.getValue());
							break;
						case 2:
							borderFlashSlider.incValue(0.1f);
							globalData->setBorderFlash(borderFlashSlider.getValue());
							break;
						case 3:
							matchLengthSlider.incValue(1.0f/14.0f);
							globalData->setMatchLength(1.0f + (matchLengthSlider.getValue() * 14.0f));
							break;
						case 4:
						case 5:
						case 6:
							keyDown(L"P2Right", L"", 0);
							break;
					}
				}
				else
				{
					switch(selectedEntry)
					{
						case 0:
							screenShakeSlider.incValue(-0.1f);
							globalData->setScreenShake(screenShakeSlider.getValue());
							break;
						case 1:
							backgroundFlashSlider.incValue(-0.1f);
							globalData->setBackgroundFlash(backgroundFlashSlider.getValue());
							break;
						case 2:
							borderFlashSlider.incValue(-0.1f);
							globalData->setBorderFlash(borderFlashSlider.getValue());
							break;
						case 3:
							matchLengthSlider.incValue(-(1.0f/14.0f));
							globalData->setMatchLength(1.0f + (matchLengthSlider.getValue() * 14.0f));
							break;
						case 4:
						case 5:
						case 6:
							keyDown(L"P2Left", L"", 0);
							break;
					}
				}
			}
			else
			{
				if(lastJoyVal.y < 0.0f)
				{
					--selectedEntry;
					if(selectedEntry < 0)
						selectedEntry = 7;
				}
				else if(lastJoyVal.y > 0.0f)
				{
					++selectedEntry;
					selectedEntry %= 8;
				}
			}

			entryTween0.reset(entries[selectedEntry][0].position);
			entryTween1.reset(entries[selectedEntry][1].position);
			entryTween2.reset(entries[selectedEntry][2].position);
			entryTween3.reset(entries[selectedEntry][3].position);
			entryTween4.reset(entries[selectedEntry][4].position);
			entryTween5.reset(entries[selectedEntry][5].position);
			entryTween6.reset(entries[selectedEntry][6].position);
			entryTween7.reset(entries[selectedEntry][7].position);

			updateSliderPositions();
			updateTextPositions();
		}
	}
	if(joyRepeat)
		--joyRepeat;

	if(!entryTween0.getFinished())
		entryTween0.tick();
	if(!entryTween1.getFinished())
		entryTween1.tick();
	if(!entryTween2.getFinished())
		entryTween2.tick();
	if(!entryTween3.getFinished())
		entryTween3.tick();
	if(!entryTween4.getFinished())
		entryTween4.tick();
	if(!entryTween5.getFinished())
		entryTween5.tick();
	if(!entryTween6.getFinished())
		entryTween6.tick();
	if(!entryTween7.getFinished())
		entryTween7.tick();

	screenShakeSlider.update();
	backgroundFlashSlider.update();
	borderFlashSlider.update();
	matchLengthSlider.update();

	if(!audioAPITween.getFinished())
		audioAPITween.tick();
	if(!redInputTween.getFinished())
		redInputTween.tick();
	if(!blueInputTween.getFinished())
		blueInputTween.tick();

	redMeter.setValue(sounder->getRedInput());
	blueMeter.setValue(sounder->getBlueInput());
}

//------------------------------------------------------------------------------
void OptionsScreen::draw(DrawerPtr& d, float timestepInterp)
{
	int i;

	d->fillRect(0.0f, d->getDrawingAreaSize(), 0.125f);

	for(i=0;i<8;++i)
	{
		d->drawText(entries[selectedEntry][i].text,
					entries[selectedEntry][i].font,
					entries[selectedEntry][i].layout,
					entryPos[i]+2.0f,
					1280.0f,
					TextLayout::Left);
		d->drawText(entries[selectedEntry][i].text,
					entries[selectedEntry][i].font,
					entries[selectedEntry][i].layout,
					entryPos[i],
					1280.0f,
					TextLayout::Left,
					1.0f,
					entries[selectedEntry][i].colour);
	}
	
	screenShakeSlider.draw(d);
	backgroundFlashSlider.draw(d);
	borderFlashSlider.draw(d);
	matchLengthSlider.draw(d);

	if(selectedEntry == 3)
	{
		std::wstringstream tempstr;
		TwoFloats tempPos(760.0f, 372.0f-32.0f);

		tempstr << (int)(1.0001f + (matchLengthSlider.getValue() * 14.0f)) << L" mins";

		d->drawText(tempstr.str(),
					L"TimeFont",
					tempPos+2.0f,
					440.0f,
					TextLayout::Centre);
		d->drawText(tempstr.str(),
					L"TimeFont",
					tempPos,
					440.0f,
					TextLayout::Centre,
					1.0f,
					1.0f);
	}

	d->drawText(audioAPIText.text,
				audioAPIText.font,
				audioAPIText.layout,
				audioAPIText.position+2.0f,
				512.0f,
				TextLayout::Left,
				1.0f);
	d->drawText(audioAPIText.text,
				audioAPIText.font,
				audioAPIText.layout,
				audioAPIText.position,
				512.0f,
				TextLayout::Left,
				1.0f,
				audioAPIText.colour);

	d->drawText(redInputText.text,
				redInputText.font,
				redInputText.layout,
				redInputText.position+2.0f,
				512.0f,
				TextLayout::Left,
				1.0f);
	d->drawText(redInputText.text,
				redInputText.font,
				redInputText.layout,
				redInputText.position,
				512.0f,
				TextLayout::Left,
				1.0f,
				redInputText.colour);

	d->drawText(blueInputText.text,
				blueInputText.font,
				blueInputText.layout,
				blueInputText.position+2.0f,
				512.0f,
				TextLayout::Left,
				1.0f);
	d->drawText(blueInputText.text,
				blueInputText.font,
				blueInputText.layout,
				blueInputText.position,
				512.0f,
				TextLayout::Left,
				1.0f,
				blueInputText.colour);

	redMeter.draw(d);
	blueMeter.draw(d);
}

//------------------------------------------------------------------------------
void OptionsScreen::keyDown(const std::wstring& action,
						    const std::wstring& keyPressed,
						    int keyModifier)
{
	std::wstringstream tempstr;
	AudioDevicePtr& audioDevice = getApplication().getAudioDevice();

	if(action == L"Escape")
		nextState(L"TitleScreen");
	else if(action == L"P2Up")
	{
		--selectedEntry;
		if(selectedEntry < 0)
			selectedEntry = 7;

		entryTween0.reset(entries[selectedEntry][0].position);
		entryTween1.reset(entries[selectedEntry][1].position);
		entryTween2.reset(entries[selectedEntry][2].position);
		entryTween3.reset(entries[selectedEntry][3].position);
		entryTween4.reset(entries[selectedEntry][4].position);
		entryTween5.reset(entries[selectedEntry][5].position);
		entryTween6.reset(entries[selectedEntry][6].position);
		entryTween7.reset(entries[selectedEntry][7].position);

		updateSliderPositions();
		updateTextPositions();
	}
	else if(action == L"P2Down")
	{
		++selectedEntry;
		selectedEntry %= 8;

		entryTween0.reset(entries[selectedEntry][0].position);
		entryTween1.reset(entries[selectedEntry][1].position);
		entryTween2.reset(entries[selectedEntry][2].position);
		entryTween3.reset(entries[selectedEntry][3].position);
		entryTween4.reset(entries[selectedEntry][4].position);
		entryTween5.reset(entries[selectedEntry][5].position);
		entryTween6.reset(entries[selectedEntry][6].position);
		entryTween7.reset(entries[selectedEntry][7].position);

		updateSliderPositions();
		updateTextPositions();
	}
	else if((action == L"P2Left") && (selectedEntry < 7))
	{
		switch(selectedEntry)
		{
			case 0:
				screenShakeSlider.incValue(-0.1f);
				globalData->setScreenShake(screenShakeSlider.getValue());
				break;
			case 1:
				backgroundFlashSlider.incValue(-0.1f);
				globalData->setBackgroundFlash(backgroundFlashSlider.getValue());
				break;
			case 2:
				borderFlashSlider.incValue(-0.1f);
				globalData->setBorderFlash(borderFlashSlider.getValue());
				break;
			case 3:
				matchLengthSlider.incValue(-(1.0f/14.0f));
				globalData->setMatchLength(1.0f + (matchLengthSlider.getValue() * 14.0f));
				break;
			case 4:
				updateAudioAPI(false);
				break;
			case 5:
				--redChannel;
				if(redChannel < 0)
				{
					--inputIndex;
					if(inputIndex < 0)
						inputIndex = audioDevice->getAvailableInputDevices().size()-1;
					sounder->setNumChannels(audioDevice->getChannelsForInputDevice(inputIndex));
					redChannel = audioDevice->getChannelsForInputDevice(inputIndex)-1;
					blueChannel = 0;
					updateBlueText();

					globalData->setAudioInputDevice(inputIndex);
					globalData->setBlueInputChannel(blueChannel);

					audioDevice->stop();
					audioDevice->setDevices(audioDevice->getAvailableInputDevices()[inputIndex],
											L"Default");
					audioDevice->setNumChannels(audioDevice->getChannelsForInputDevice(inputIndex), 2);
					audioDevice->start();
				}
				updateRedText();
				globalData->setRedInputChannel(redChannel);
				sounder->setRedInputChannel(redChannel);

				tempstr << L"< " << redInput.substr(0, 19) << L" " << (redChannel+1) << L" >";
				redInputText.text = tempstr.str();
				redInputText.layout.clear();
				break;
			case 6:
				--blueChannel;
				if(blueChannel < 0)
					blueChannel = audioDevice->getChannelsForInputDevice(inputIndex)-1;
				updateBlueText();
				globalData->setBlueInputChannel(blueChannel);
				sounder->setBlueInputChannel(blueChannel);

				tempstr << L"< " << blueInput.substr(0, 19) << L" " << (blueChannel+1) << L" >";
				blueInputText.text = tempstr.str();
				blueInputText.layout.clear();
				
				break;
		}
	}
	else if((action == L"P2Right") && (selectedEntry < 7))
	{
		switch(selectedEntry)
		{
			case 0:
				screenShakeSlider.incValue(0.1f);
				globalData->setScreenShake(screenShakeSlider.getValue());
				break;
			case 1:
				backgroundFlashSlider.incValue(0.1f);
				globalData->setBackgroundFlash(backgroundFlashSlider.getValue());
				break;
			case 2:
				borderFlashSlider.incValue(0.1f);
				globalData->setBorderFlash(borderFlashSlider.getValue());
				break;
			case 3:
				matchLengthSlider.incValue(1.0f/14.0f);
				globalData->setMatchLength(1.0f + (matchLengthSlider.getValue() * 14.0f));
				break;
			case 4:
				updateAudioAPI(true);
				break;
			case 5:
				++redChannel;
				if(redChannel >= audioDevice->getChannelsForInputDevice(inputIndex))
				{
					++inputIndex;
					inputIndex %= audioDevice->getAvailableInputDevices().size();
					sounder->setNumChannels(audioDevice->getChannelsForInputDevice(inputIndex));
					redChannel = 0;
					blueChannel = 0;
					updateBlueText();

					globalData->setAudioInputDevice(inputIndex);
					globalData->setBlueInputChannel(blueChannel);

					audioDevice->stop();
					audioDevice->setDevices(audioDevice->getAvailableInputDevices()[inputIndex],
											L"Default");
					audioDevice->setNumChannels(audioDevice->getChannelsForInputDevice(inputIndex), 2);
					audioDevice->start();
				}
				updateRedText();
				sounder->setRedInputChannel(redChannel);

				globalData->setRedInputChannel(redChannel);

				tempstr << L"< " << redInput.substr(0, 19) << L" " << (redChannel+1) << L" >";
				redInputText.text = tempstr.str();
				redInputText.layout.clear();
				break;
			case 6:
				++blueChannel;
				blueChannel %= audioDevice->getChannelsForInputDevice(inputIndex);
				updateBlueText();
				sounder->setBlueInputChannel(blueChannel);

				globalData->setBlueInputChannel(blueChannel);

				tempstr << L"< " << blueInput.substr(0, 19) << L" " << (blueChannel+1) << L" >";
				blueInputText.text = tempstr.str();
				blueInputText.layout.clear();
				break;
		}
	}
	else if((action == L"Action") || (action == L"P1Kick"))
	{
		if(selectedEntry == 7)
			nextState(L"TitleScreen");
	}
}

//------------------------------------------------------------------------------
void OptionsScreen::joyMove(const std::wstring& action,
						    float value,
						    int joystick,
						    int axis)
{
	if(action == L"P1Up")
	{
		if(fabsf(value) < 0.25f)
		{
			lastJoyVal.y = 0.0f;
			if(lastJoyVal.x == 0.0f)
				joyRepeat = 0;
		}
		else
			lastJoyVal.y = value;
	}
	else if(action == L"P1Down")
	{
		if(fabsf(value) < 0.25f)
		{
			lastJoyVal.y = 0.0f;
			if(lastJoyVal.x == 0.0f)
				joyRepeat = 0;
		}
		else
			lastJoyVal.y = value;
	}
	else if(action == L"P1Left")
	{
		if(fabsf(value) < 0.25f)
		{
			lastJoyVal.x = 0.0f;
			if(lastJoyVal.y == 0.0f)
				joyRepeat = 0;
		}
		else
			lastJoyVal.x = value;
	}
	else if(action == L"P1Right")
	{
		if(fabsf(value) < 0.25f)
		{
			lastJoyVal.x = 0.0f;
			if(lastJoyVal.y == 0.0f)
				joyRepeat = 0;
		}
		else
			lastJoyVal.x = value;
	}
}

//------------------------------------------------------------------------------
void OptionsScreen::updateSliderPositions()
{
	switch(selectedEntry)
	{
		case 0:
			screenShakeSlider.setBig(true);
			screenShakeSlider.setPosition(TwoFloats(760.0f, 160.0f));
			if(lastSelected == 1)
			{
				backgroundFlashSlider.setBig(false);
				backgroundFlashSlider.setPosition(TwoFloats(760.0f, 232.0f));
			}
			else if(lastSelected == 7)
			{
				backgroundFlashSlider.setBig(false);
				backgroundFlashSlider.setPosition(TwoFloats(760.0f, 232.0f));
				borderFlashSlider.setBig(false);
				borderFlashSlider.setPosition(TwoFloats(760.0f, 302.0f));
				matchLengthSlider.setBig(false);
				matchLengthSlider.setPosition(TwoFloats(760.0f, 372.0f));
			}
			break;
		case 1:
			backgroundFlashSlider.setBig(true);
			backgroundFlashSlider.setPosition(TwoFloats(760.0f, 232.0f-8.0f));
			if(lastSelected == 0)
			{
				screenShakeSlider.setBig(false);
				screenShakeSlider.setPosition(TwoFloats(760.0f, 126.0f));
			}
			else if(lastSelected == 2)
			{
				borderFlashSlider.setBig(false);
				borderFlashSlider.setPosition(TwoFloats(760.0f, 302.0f));
			}
			break;
		case 2:
			borderFlashSlider.setBig(true);
			borderFlashSlider.setPosition(TwoFloats(760.0f, 302.0f-8.0f));
			if(lastSelected == 1)
			{
				backgroundFlashSlider.setBig(false);
				backgroundFlashSlider.setPosition(TwoFloats(760.0f, 232.0f-48.0f));
			}
			else if(lastSelected == 3)
			{
				matchLengthSlider.setBig(false);
				matchLengthSlider.setPosition(TwoFloats(760.0f, 372.0f));
			}
			break;
		case 3:
			matchLengthSlider.setBig(true);
			matchLengthSlider.setPosition(TwoFloats(760.0f, 372.0f-8.0f));
			if(lastSelected == 2)
			{
				borderFlashSlider.setBig(false);
				borderFlashSlider.setPosition(TwoFloats(760.0f, 302.0f-48.0f));
			}
			break;
		case 4:
			if(lastSelected == 3)
			{
				matchLengthSlider.setBig(false);
				matchLengthSlider.setPosition(TwoFloats(760.0f, 372.0f-48.0f));
			}
			break;
		case 7:
			if(lastSelected == 0)
			{
				screenShakeSlider.setBig(false);
				screenShakeSlider.setPosition(TwoFloats(760.0f, 126.0f));
				backgroundFlashSlider.setBig(false);
				backgroundFlashSlider.setPosition(TwoFloats(760.0f, 232.0f-48.0f));
				borderFlashSlider.setBig(false);
				borderFlashSlider.setPosition(TwoFloats(760.0f, 302.0f-48.0f));
				matchLengthSlider.setBig(false);
				matchLengthSlider.setPosition(TwoFloats(760.0f, 372.0f-48.0f));
			}
			break;
	}
}

//------------------------------------------------------------------------------
void OptionsScreen::updateTextPositions()
{
	switch(selectedEntry)
	{
		case 0:
			if(lastSelected == 7)
			{
				audioAPITween.reset(TwoFloats(audioAPIText.position.x, 442.0f));
				redInputTween.reset(TwoFloats(redInputText.position.x, 512.0f));
				blueInputTween.reset(TwoFloats(blueInputText.position.x, 582.0f));
			}
			break;
		case 1:
			break;
		case 2:
			break;
		case 3:
			if(lastSelected == 4)
			{
				audioAPIText.text = audioAPI;
				audioAPIText.layout.clear();

				audioAPITween.reset(TwoFloats(audioAPIText.position.x, 442.0f));
			}
			break;
		case 4:
			{
				std::wstringstream tempstr;

				tempstr << L"< " << audioAPI << L" >";
				audioAPIText.text = tempstr.str();
				audioAPIText.layout.clear();
				audioAPITween.reset(TwoFloats(audioAPIText.position.x, 442.0f-32.0f));
			}
			if(lastSelected == 5)
			{
				redInputText.text = redInput;
				redInputText.layout.clear();

				redInputTween.reset(TwoFloats(redInputText.position.x, 512.0f));
			}
			break;
		case 5:
			{
				std::wstringstream tempstr;

				tempstr << L"< " << redInput.substr(0, 19) << L" " << (redChannel+1) << L" >";
				redInputText.text = tempstr.str();
				redInputText.layout.clear();
				redInputTween.reset(TwoFloats(audioAPIText.position.x, 512.0f-32.0f));
			}
			if(lastSelected == 4)
			{
				audioAPIText.text = audioAPI;
				audioAPIText.layout.clear();

				audioAPITween.reset(TwoFloats(redInputText.position.x, 442.0f-48.0f));
			}
			else if(lastSelected == 6)
			{
				blueInputText.text = blueInput;
				blueInputText.layout.clear();

				blueInputTween.reset(TwoFloats(blueInputText.position.x, 582.0f));
			}
			break;
		case 6:
			{
				std::wstringstream tempstr;

				tempstr << L"< " << blueInput.substr(0, 19) << L" " << (blueChannel+1) << L" >";
				blueInputText.text = tempstr.str();
				blueInputText.layout.clear();
				blueInputTween.reset(TwoFloats(blueInputText.position.x, 582.0f-32.0f));
			}
			if(lastSelected == 5)
			{
				redInputText.text = redInput;
				redInputText.layout.clear();

				redInputTween.reset(TwoFloats(redInputText.position.x, 512.0f-48.0f));
			}
			break;
		case 7:
			if(lastSelected == 0)
			{
				audioAPITween.reset(TwoFloats(audioAPIText.position.x, 442.0f-48.0f));
				redInputTween.reset(TwoFloats(redInputText.position.x, 512.0f-48.0f));
				blueInputTween.reset(TwoFloats(blueInputText.position.x, 582.0f-48.0f));
			}
			else if(lastSelected == 6)
			{
				blueInputText.text = blueInput;
				blueInputText.layout.clear();

				blueInputTween.reset(TwoFloats(blueInputText.position.x, 582.0f-48.0f));
			}
			break;
	}
	lastSelected = selectedEntry;
}

//------------------------------------------------------------------------------
void OptionsScreen::updateAudioAPI(bool dir)
{
	int i;
	int index;
	std::wstringstream tempstr;
	bool availableInputDevice = false;
	AudioDevicePtr& audioDevice = getApplication().getAudioDevice();
	const std::vector<std::wstring>& apiList = audioDevice->getAvailableAPIs();

	for(i=0;i<(int)apiList.size();++i)
	{
		if(audioAPI == apiList[i])
		{
			index = i;
			break;
		}
	}
	if(dir)
	{
		++index;
		index %= apiList.size();
	}
	else
	{
		--index;
		if(index < 0)
			index = apiList.size()-1;
	}

	audioDevice->stop();
	audioDevice->setAPI(apiList[index]);

	audioAPI = audioDevice->getAPI();

	if(audioDevice->getAvailableInputDevices().size() > 0)
		availableInputDevice = true;

	if(availableInputDevice)
		sounder->setNumChannels(audioDevice->getChannelsForInputDevice(0));
	else
		sounder->setNumChannels(0);
	
	inputIndex = 0;
	redChannel = 0;
	blueChannel = 0;
	updateRedText();
	updateBlueText();
	globalData->setAudioAPI(audioAPI);
	globalData->setAudioInputDevice(inputIndex);

	tempstr << L"< " << audioAPI << L" >";
	audioAPIText.text = tempstr.str();
	audioAPIText.layout.clear();
	
	if(availableInputDevice)
	{
		audioDevice->setDevices(audioDevice->getAvailableInputDevices()[inputIndex],
								L"Default");
		audioDevice->setNumChannels(audioDevice->getChannelsForInputDevice(inputIndex), 2);
	}
	else
	{
		audioDevice->setDevices(L"Default", L"Default");
		audioDevice->setNumChannels(0, 2);
	}
	audioDevice->start();
}

//------------------------------------------------------------------------------
void OptionsScreen::updateRedText()
{
	std::wstringstream tempstr;
	AudioDevicePtr& audioDevice = getApplication().getAudioDevice();

	tempstr << audioDevice->getAvailableInputDevices()[inputIndex].substr(0, 22);
	tempstr << L" " << redChannel+1;
	redInput = tempstr.str();

	redInputText.text = redInput;
	redInputText.layout.clear();
}

//------------------------------------------------------------------------------
void OptionsScreen::updateBlueText()
{
	std::wstringstream tempstr;
	AudioDevicePtr& audioDevice = getApplication().getAudioDevice();

	tempstr << audioDevice->getAvailableInputDevices()[inputIndex].substr(0, 22);
	tempstr << L" " << blueChannel+1;
	blueInput = tempstr.str();

	blueInputText.text = blueInput;
	blueInputText.layout.clear();
}
