//	ScreamingSnakeballGlobalData.h - The game's global data.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SCREAMINGSNAKEBALLGLOBALDATA_H_
#define SCREAMINGSNAKEBALLGLOBALDATA_H_

#include "OptionsFile.h"
#include "GlobalData.h"

///	The game's global data.
class ScreamingSnakeballGlobalData : public GlobalData
{
  public:
	///	Constructor.
	ScreamingSnakeballGlobalData();
	///	Destructor.
	~ScreamingSnakeballGlobalData();

	///	Sets the amount of screen shake to use.
	void setScreenShake(float val);
	///	Sets how much to flash the background.
	void setBackgroundFlash(float val);
	///	Sets how much to flash the border.
	void setBorderFlash(float val);
	///	Sets the match length
	void setMatchLength(float val);
	///	Sets the audio API we're using.
	void setAudioAPI(const std::wstring& val);
	///	Sets the audio input device index.
	void setAudioInputDevice(int val);
	///	Sets the red input channel.
	void setRedInputChannel(int val);
	///	Sets the blue input channel.
	void setBlueInputChannel(int val);

	///	Sets whispering mode.
	void setWhispering(bool val);
	///	Sets shouting mode.
	void setShouting(bool val);

	///	Returns the amount of screen shake to use.
	float getScreenShake() const {return screenShake;};
	///	Returns how much to flash the background.
	float getBackgroundFlash() const {return backgroundFlash;};
	///	Returns how much to flash the border.
	float getBorderFlash() const {return borderFlash;};
	///	Returns the match length in minutes.
	float getMatchLength() const {return matchLength;};
	///	Returns the audio API we're using.
	const std::wstring& getAudioAPI() const {return audioAPI;};
	///	Returns the audio input device index.
	int getAudioInputDevice() const {return audioInputDevice;};
	///	Returns the red input channel.
	int getRedInputChannel() const {return redInputChannel;};
	///	Returns the blue input channel.
	int getBlueInputChannel() const {return blueInputChannel;};

	///	Returns true if we're playing in whispering mode.
	bool getWhispering() const {return whispering;};
	///	Returns true if we're playing in shouting mode.
	bool getShouting() const {return shouting;};
  private:
	///	The amount of screen shake to use.
	float screenShake;
	///	How much to flash the background.
	float backgroundFlash;
	///	How much to flash the border.
	float borderFlash;
	///	The match length in minutes.
	float matchLength;
	///	The audio API we're using.
	std::wstring audioAPI;
	///	The audio input device index.
	int audioInputDevice;
	///	The red input channel.
	int redInputChannel;
	///	The blue input channel.
	int blueInputChannel;

	///	True if we're playing in whispering mode.
	bool whispering;
	///	True if we're playing in shouting mode.
	bool shouting;

	///	Our options file.
	OptionsFile optionsFile;
};

#endif
