//	TitleScreen.cpp - The title screen state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ScreamingSnakeballGlobalData.h"
#include "sound/AudioDevice.h"
#include "SnakeballSounder.h"
#include "drawing/Drawer.h"
#include "TitleScreen.h"
#include "Application.h"

//------------------------------------------------------------------------------
GameStateCreator<TitleScreen> TitleScreen::creator;
bool TitleScreen::firstRun = true;

//------------------------------------------------------------------------------
TitleScreen::TitleScreen(Application& app, SounderPtr& sound):
GameState(app, sound),
entryTween0(entryPos[0], entryPos[0], 0.1f),
entryTween1(entryPos[1], entryPos[1], 0.1f),
entryTween2(entryPos[2], entryPos[2], 0.1f),
entryTween3(entryPos[3], entryPos[3], 0.1f),
entryTween4(entryPos[4], entryPos[4], 0.1f),
selectedEntry(0),
lastJoyVal(0.0f),
joyRepeat(0),
fadeIn(1.0f),
fadeInTween(fadeIn, 0.0f, 0.05f),
fadeOutTween(fadeIn, 0.0f, 0.02f)
{
	int i;

	globalData = dynamic_pointer_cast<ScreamingSnakeballGlobalData>(getGlobalData());

	fadeOutTween.setIndex(1.0f);

	if(firstRun)
	{
		int inputIndex;
		std::wstring storedAPI;
		bool availableInputDevice = false;
		AudioDevicePtr& audioDevice = getApplication().getAudioDevice();
		shared_ptr<SnakeballSounder> sounder = dynamic_pointer_cast<SnakeballSounder>(sound);

		firstRun = false;

		storedAPI = globalData->getAudioAPI();
		inputIndex = globalData->getAudioInputDevice();

		audioDevice->stop();
		audioDevice->setAPI(storedAPI);

		if(audioDevice->getAvailableInputDevices().size() > 0)
			availableInputDevice = true;

		if(availableInputDevice)
			sounder->setNumChannels(audioDevice->getChannelsForInputDevice(inputIndex));
		else
			sounder->setNumChannels(0);
		sounder->setRedInputChannel(globalData->getRedInputChannel());
		sounder->setBlueInputChannel(globalData->getBlueInputChannel());
	
		if(availableInputDevice)
		{
			audioDevice->setDevices(audioDevice->getAvailableInputDevices()[inputIndex],
									L"Default");
			audioDevice->setNumChannels(audioDevice->getChannelsForInputDevice(inputIndex), 2);
		}
		else
		{
			audioDevice->setDevices(L"Default", L"Default");
			audioDevice->setNumChannels(0, 2);
		}
		audioDevice->start();
	}
	else
	{
		fadeIn = 0.0f;
		fadeInTween.setIndex(1.0f);
	}

	for(i=0;i<5;++i)
	{
		entries[i][0].colour = ThreeFloats(1.0f, 0.5f, 0.5f);
		entries[i][0].text = L"Play Screaming Snakeball";
		entries[i][1].colour = ThreeFloats(0.90625f, 0.5f, 0.5625f);
		entries[i][1].text = L"Play Shouting Snakeball";
		entries[i][2].colour = ThreeFloats(0.8125f, 0.5f, 0.625f);
		entries[i][2].text = L"Play Whispering Snakeball";
		entries[i][3].colour = ThreeFloats(0.75f, 0.5f, 0.75f);
		entries[i][3].text = L"Options";
		entries[i][4].colour = ThreeFloats(0.52734375f, 0.5f, 1.0f);
		entries[i][4].text = L"Quit";
	}

	entries[0][0].position = TwoFloats(70.0f, 364.0f);
	entries[0][0].font = L"ScoreFont";
	entries[0][1].position = TwoFloats(70.0f, 450.0f);
	entries[0][1].font = L"TimeFont";
	entries[0][2].position = TwoFloats(70.0f, 520.0f);
	entries[0][2].font = L"TimeFont";
	entries[0][3].position = TwoFloats(70.0f, 590.0f);
	entries[0][3].font = L"TimeFont";
	entries[0][4].position = TwoFloats(70.0f, 664.0f);
	entries[0][4].font = L"TimeFont";

	entries[1][0].position = TwoFloats(70.0f, 380.0f-54.0f);
	entries[1][0].font = L"TimeFont";
	entries[1][1].position = TwoFloats(70.0f, 450.0f-8.0f);
	entries[1][1].font = L"ScoreFont";
	entries[1][2].position = TwoFloats(70.0f, 520.0f);
	entries[1][2].font = L"TimeFont";
	entries[1][3].position = TwoFloats(70.0f, 590.0f);
	entries[1][3].font = L"TimeFont";
	entries[1][4].position = TwoFloats(70.0f, 664.0f);
	entries[1][4].font = L"TimeFont";

	entries[2][0].position = TwoFloats(70.0f, 380.0f-48.0f);
	entries[2][0].font = L"TimeFont";
	entries[2][1].position = TwoFloats(70.0f, 450.0f-54.0f);
	entries[2][1].font = L"TimeFont";
	entries[2][2].position = TwoFloats(70.0f, 520.0f-8.0f);
	entries[2][2].font = L"ScoreFont";
	entries[2][3].position = TwoFloats(70.0f, 590.0f);
	entries[2][3].font = L"TimeFont";
	entries[2][4].position = TwoFloats(70.0f, 664.0f);
	entries[2][4].font = L"TimeFont";

	entries[3][0].position = TwoFloats(70.0f, 380.0f-48.0f);
	entries[3][0].font = L"TimeFont";
	entries[3][1].position = TwoFloats(70.0f, 450.0f-48.0f);
	entries[3][1].font = L"TimeFont";
	entries[3][2].position = TwoFloats(70.0f, 520.0f-48.0f);
	entries[3][2].font = L"TimeFont";
	entries[3][3].position = TwoFloats(70.0f, 590.0f-8.0f);
	entries[3][3].font = L"ScoreFont";
	entries[3][4].position = TwoFloats(70.0f, 664.0f);
	entries[3][4].font = L"TimeFont";

	entries[4][0].position = TwoFloats(70.0f, 380.0f-48.0f);
	entries[4][0].font = L"TimeFont";
	entries[4][1].position = TwoFloats(70.0f, 450.0f-48.0f);
	entries[4][1].font = L"TimeFont";
	entries[4][2].position = TwoFloats(70.0f, 520.0f-48.0f);
	entries[4][2].font = L"TimeFont";
	entries[4][3].position = TwoFloats(70.0f, 590.0f-48.0f);
	entries[4][3].font = L"TimeFont";
	entries[4][4].position = TwoFloats(70.0f, 664.0f-8.0f);
	entries[4][4].font = L"ScoreFont";

	for(i=0;i<5;++i)
		entryPos[i] = entries[0][i].position;
	entryTween0.setIndex(1.0f);
	entryTween1.setIndex(1.0f);
	entryTween2.setIndex(1.0f);
	entryTween3.setIndex(1.0f);
	entryTween4.setIndex(1.0f);
}

//------------------------------------------------------------------------------
TitleScreen::~TitleScreen()
{

}

//------------------------------------------------------------------------------
void TitleScreen::update()
{
	if(lastJoyVal != 0.0f)
	{
		if(!joyRepeat)
		{
			joyRepeat = 15;

			if(lastJoyVal < 0.0f)
			{
				--selectedEntry;
				if(selectedEntry < 0)
					selectedEntry = 4;
			}
			else if(lastJoyVal > 0.0f)
			{
				++selectedEntry;
				selectedEntry %= 5;
			}

			entryTween0.reset(entries[selectedEntry][0].position);
			entryTween1.reset(entries[selectedEntry][1].position);
			entryTween2.reset(entries[selectedEntry][2].position);
			entryTween3.reset(entries[selectedEntry][3].position);
			entryTween4.reset(entries[selectedEntry][4].position);
		}
	}
	if(joyRepeat)
		--joyRepeat;

	if(!entryTween0.getFinished())
		entryTween0.tick();
	if(!entryTween1.getFinished())
		entryTween1.tick();
	if(!entryTween2.getFinished())
		entryTween2.tick();
	if(!entryTween3.getFinished())
		entryTween3.tick();
	if(!entryTween4.getFinished())
		entryTween4.tick();

	if(!fadeInTween.getFinished())
		fadeInTween.tick();
	if(!fadeOutTween.getFinished())
	{
		fadeOutTween.tick();
		if(fadeOutTween.getFinished())
			quit();
	}
}

//------------------------------------------------------------------------------
void TitleScreen::draw(DrawerPtr& d, float timestepInterp)
{
	int i;

	d->fillRect(0.0f, d->getDrawingAreaSize(), 0.125f);

	d->drawImage(L"TitleText0", TwoFloats(71.0f, 38.0f));
	d->drawImage(L"TitleText1", TwoFloats(684.0f, 38.0f));

	for(i=0;i<5;++i)
	{
		d->drawText(entries[selectedEntry][i].text,
					entries[selectedEntry][i].font,
					entries[selectedEntry][i].layout,
					//entries[selectedEntry][i].position+2.0f,
					entryPos[i]+2.0f,
					1280.0f,
					TextLayout::Left);
		d->drawText(entries[selectedEntry][i].text,
					entries[selectedEntry][i].font,
					entries[selectedEntry][i].layout,
					//entries[selectedEntry][i].position,
					entryPos[i],
					1280.0f,
					TextLayout::Left,
					1.0f,
					entries[selectedEntry][i].colour);
	}

	if(fadeIn > 0.0f)
		d->fillRect(0.0f, d->getDrawingAreaSize(), 0.125f, fadeIn);
}

//------------------------------------------------------------------------------
void TitleScreen::keyDown(const std::wstring& action,
						  const std::wstring& keyPressed,
						  int keyModifier)
{
	if(action == L"Escape")
		fadeOutTween.reset(1.0f);
	else if(action == L"P2Up")
	{
		--selectedEntry;
		if(selectedEntry < 0)
			selectedEntry = 4;

		entryTween0.reset(entries[selectedEntry][0].position);
		entryTween1.reset(entries[selectedEntry][1].position);
		entryTween2.reset(entries[selectedEntry][2].position);
		entryTween3.reset(entries[selectedEntry][3].position);
		entryTween4.reset(entries[selectedEntry][4].position);
	}
	else if(action == L"P2Down")
	{
		++selectedEntry;
		selectedEntry %= 5;

		entryTween0.reset(entries[selectedEntry][0].position);
		entryTween1.reset(entries[selectedEntry][1].position);
		entryTween2.reset(entries[selectedEntry][2].position);
		entryTween3.reset(entries[selectedEntry][3].position);
		entryTween4.reset(entries[selectedEntry][4].position);
	}
	else if((action == L"Action") || (action == L"P1Kick"))
	{
		switch(selectedEntry)
		{
			case 0:
				globalData->setWhispering(false);
				globalData->setShouting(false);
				nextState(L"GamePlayState");
				break;
			case 1:
				globalData->setWhispering(false);
				globalData->setShouting(true);
				nextState(L"GamePlayState");
				break;
			case 2:
				globalData->setWhispering(true);
				globalData->setShouting(false);
				nextState(L"GamePlayState");
				break;
			case 3:
				nextState(L"OptionsScreen");
				break;
			case 4:
				fadeOutTween.reset(1.0f);
				break;
		}
	}
}

//------------------------------------------------------------------------------
void TitleScreen::joyMove(const std::wstring& action,
						  float value,
						  int joystick,
						  int axis)
{
	if(action == L"P1Up")
	{
		if(fabsf(value) < 0.25f)
		{
			lastJoyVal = 0.0f;
			joyRepeat = 0;
		}
		else
			lastJoyVal = value;
	}
	else if(action == L"P1Down")
	{
		if(fabsf(value) < 0.25f)
		{
			lastJoyVal = 0.0f;
			joyRepeat = 0;
		}
		else
			lastJoyVal = value;
	}
}
